<?php
/**
 * Daisycon affiliatemarketing plugin config
 *
 * @category  Daisycon
 * @package   Daisycon_Affiliatemarketing
 * @copyright Copyright (c) 2018 Daisycon B.V.
 * @author    Daisycon B.V. <support@daisycon.com>
 */
abstract class Daisycon_Affiliatemarketing_Model_Api_Resource_Abstract implements \JsonSerializable, \ArrayAccess
{
    /**
     * The current connection
     *
     * @var Daisycon_Affiliatemarketing_Model_Api_Connection
     */
    protected $_connection;

    /**
     * The model's attributes and values
     *
     * @var array
     */
    protected $_attributes = array();

    /**
     * The model's assignable attributes
     *
     * @var array
     */
    protected $_assignable = array();

    /**
     * The service url of this model
     *
     * @var string
     */
    protected $_url = '';

    /**
     * @var string Name of the primary key for this model
     */
    protected $_primaryKey = 'id';

    /**
     * Daisycon_Affiliatemarketing_Model_Api_Resource_Abstract constructor.
     *
     * @param Daisycon_Affiliatemarketing_Model_Api_Connection $connection
     * @param array $attributes
     */
    public function __construct(
        Daisycon_Affiliatemarketing_Model_Api_Connection $connection,
        array $attributes = array()
    ) {
        $this->_connection = $connection;
        $this->_assign($attributes);
    }

    /**
     * Get the connection instance
     *
     * @return Daisycon_Affiliatemarketing_Model_Api_Connection
     */
    public function getConnection()
    {
        return $this->_connection;
    }

    /**
     * Get the model's attributes
     *
     * @return array
     */
    public function getAttributes()
    {
        return $this->_attributes;
    }

    /**
     * Get the model's service url
     *
     * @return string
     */
    public function getUrl()
    {
        return $this->_url;
    }

    /**
     * Get the model's primary key
     *
     * @return string
     */
    public function getPrimaryKey()
    {
        return $this->_primaryKey;
    }

    /**
     * Get the model's primary key value
     *
     * @return mixed
     */
    public function getPrimaryKeyValue()
    {
        return $this->__get($this->_primaryKey);
    }

    /**
     * Fill the entity from an array
     *
     * @param array $attributes
     */
    protected function _assign(array $attributes)
    {
        $attributes = array_intersect_key($attributes, array_flip($this->_assignable));
        foreach ($attributes as $key => $value) {
            if (true === $this->isAttribute($key)) {
                $this->setAttribute($key, $value);
            }
        }
    }

    /**
     * Is assignable property
     *
     * @param string $key
     * @return bool
     */
    protected function isAttribute($key)
    {
        return true === in_array($key, $this->_assignable);
    }

    /**
     * Set a attribute
     *
     * @param string $key
     * @param mixed $value
     * @return $this
     */
    protected function setAttribute($key, $value)
    {
        $this->_attributes[$key] = $value;
        return $this;
    }

    /**
     * Get a value
     *
     * @param string $key
     * @return mixed|null
     */
    public function __get($key)
    {
        if (true === isset($this->_attributes[$key])) {
            return $this->_attributes[$key];
        }

        return null;
    }

    /**
     * Set a value
     *
     * @param string $key
     * @param mixed $value
     * @return $this
     */
    public function __set($key, $value)
    {
        if (true === $this->isAttribute($key)) {
            $this->setAttribute($key, $value);
        }

        return $this;
    }

    /**
     * Check if attribute is set
     *
     * @param string $name
     * @return bool
     */
    public function __isset($name)
    {
        return null !== $this->__get($name);
    }

    /**
     * Allow Get/Set methods
     *
     * @param string $method
     * @param array $args
     * @return mixed
     * @throws Mage_Core_Exception
     */
    public function __call($method, $args)
    {
        $prefix = substr($method, 0, 3);
        $attribute = lcfirst(substr($method, 3));
        switch ($prefix)
        {
            case 'get':
                return $this->__get($attribute);

            case 'set':
                return $this->__set($attribute, true === isset($args[0]) ? $args[0] : null);

            default:
                throw new Mage_Core_Exception(
                    'Invalid method: ' .  get_class($this) . '::' . $method
                );
        }

    }

    /**
     * Checks if primaryKey holds a value
     *
     * @return boolean
     */
    public function exists()
    {
        if (false === array_key_exists($this->_primaryKey, $this->_attributes)) {
            return false;
        }

        return false === empty($this->_attributes[$this->_primaryKey]);
    }

    /**
     * Return the JSON format of the data
     *
     * @param int $options http://php.net/manual/en/json.constants.php
     * @return string
     */
    public function json($options = 0)
    {
        return json_encode($this->jsonSerialize(), $options);
    }

    /**
     * Return serializable data
     *
     * @return array
     */
    public function jsonSerialize()
    {
        return $this->_attributes;
    }

    /**
     * Find a single resource
     *
     * @param int $id
     * @return Daisycon_Affiliatemarketing_Model_Api_Resource_Abstract|null
     */
    public function one($id)
    {
        $result = $this->getConnection()->get($this->getUrl() . '/' . $id);

        if (true === empty($result)) {
            return null;
        }

        return new static($this->getConnection(), $result);
    }

    /**
     * Query all items
     *
     * @param array $filter
     * @return array
     */
    public function all(array $filter = array())
    {
        $result = $this->getConnection()->get($this->getUrl(), $filter);

        return $this->collectionFromResult($result);
    }

    /**
     * Convert the api response to a collection of models
     *
     * @param array $results
     * @return array
     */
    protected function collectionFromResult($results)
    {
        if (true === empty($results)) {
            return array();
        }

        $singular = array_filter(array_keys($results), 'is_string');
        if (false === empty($singular)) {
            $results = array($results);
        }

        $connection = $this->getConnection();
        while (null !== $this->getConnection()->getNextUrl()) {
            $nextResult = $connection->get($connection->getNextUrl());

            $singular = array_filter(array_keys($nextResult), 'is_string');
            if (false === empty($singular)) {
                $nextResult = array($nextResult);
            }

            $results = array_merge($results, $nextResult);
        }

        $collection = array();
        foreach ($results as $result) {
            $collection[] = new static($this->getConnection(), $result);
        }

        return $collection;
    }

    /**
     * Implementation of ArrayAccess::offsetSet()
     * 
     * @param string $offset
     * @param mixed $value
     * @return void
     */
    public function offsetSet($offset, $value)
    {
        if (null !== $offset) {
            $this->_attributes[$offset] = $value;
        }

        return null;
    }

    /**
     * Implementation of ArrayAccess::offsetExists()
     *
     * @param string $offset
     * @return bool
     */
    public function offsetExists($offset)
    {
        return true === isset($this->_attributes[$offset]);
    }
    
    /**
     * Implementation of ArrayAccess::offsetUnset()
     *
     * @param string $offset
     * @return void
     */
    public function offsetUnset($offset)
    {
        unset($this->_attributes[$offset]);
    }
    
    /**
     * Implementation of ArrayAccess::offsetGet()
     *
     * @param string $offset
     * @return mixed
     */
    public function offsetGet($offset)
    {
        return true === isset($this->_attributes[$offset]) ? $this->_attributes[$offset] : null;
    }

}
